SYMBOL_TYPE_SPOT = "SPOT"

ORDER_STATUS_NEW = "NEW"
ORDER_STATUS_PARTIALLY_FILLED = "PARTIAL_FILLED"
ORDER_STATUS_FILLED = "FILLED"
ORDER_STATUS_CANCELED = "CANCELLED"
ORDER_STATUS_REJECTED = "REJECTED"
ORDER_STATUS_INCOMPLETE = "INCOMPLETE"
ORDER_STATUS_COMPLETED = "COMPLETED"

KLINE_INTERVAL_1MINUTE = "1m"
KLINE_INTERVAL_5MINUTE = "5m"
KLINE_INTERVAL_15MINUTE = "15m"
KLINE_INTERVAL_30MINUTE = "30m"
KLINE_INTERVAL_1HOUR = "1h"
KLINE_INTERVAL_1DAY = "1d"
KLINE_INTERVAL_1WEEK = "1w"
KLINE_INTERVAL_1MONTH = "1M"

SIDE_BUY = "BUY"
SIDE_SELL = "SELL"

ORDER_TYPE_LIMIT = "LIMIT"
ORDER_TYPE_MARKET = "MARKET"
ORDER_TYPE_IOC = "IOC"
ORDER_TYPE_FOK = "FOK"
ORDER_TYPE_POST_ONLY = "POST_ONLY"
ORDER_TYPE_ASK = "ASK"
ORDER_TYPE_BID = "BID"
